
varying vec3 N;
varying vec3 V;

varying float safeShiny;
varying vec3 Lm;

//#define MULTITYPE  
#define AMBIENT_ALPHA_TAG

// watch out for pow(x,y), invalid for [x < 0] or ([x == 0] && [y <= 0])


uniform sampler2D bzTexture0;
uniform sampler2D bzTexture1;


void main (void)
{
  const int myMaxLights = 5;

  // do the texture fetch first
//  const vec4 texcoords = gl_TextureMatrix[0] * gl_TexCoord[0];
//  const vec4 Itext = texture2D(bzTexture0, vec2(texcoords));
  const vec4 Itext0 = texture2D(bzTexture0, vec2(gl_TexCoord[0]));
  const vec4 Itext1 = texture2D(bzTexture1, vec2(gl_TexCoord[1]));

  // light vectors
  const vec3 E = normalize(-V); // eye coordinates
  
#ifdef MULTITYPE
  vec3 Idiff = vec3(0, 0, 0);
  vec3 Ispec = vec3(0, 0, 0);
#endif
//  const float safeShiny = max(gl_FrontMaterial.shininess, 0.001);
//  const float safeShiny = gl_FrontMaterial.shininess + 0.001;
  
  

  // the main light
#ifndef MULTITYPE
//  const vec3 Lm = normalize(gl_LightSource[0].position.xyz);
  const vec3 Rm = normalize(-reflect(Lm, N));
  
    // diffuse
  vec3 Idiff = gl_FrontMaterial.diffuse.rgb * gl_LightSource[0].diffuse.rgb 
               * max(dot(N, Lm), 0.0);

    // specular
  vec3 Ispec = gl_FrontMaterial.specular.rgb * gl_LightSource[0].specular.rgb
               * pow(max(dot(Rm, E), 0.0), safeShiny); //FIXME
#endif // MULTITYPE


#ifdef MULTITYPE
  for (int i = 0; i < myMaxLights; i++) {
#else   
  for (int i = 1; i < myMaxLights; i++) {
#endif // MULTITYPE

#ifdef AMBIENT_ALPHA_TAG  
    if (gl_LightSource[i].ambient.a != 0.0) {
#endif    

//    if (gl_LightSource[i].spotCutoff < 1.0) {
//      continue;
//    }

#ifdef MULTITYPE
//    if (gl_LightSource[i].spotCutoff < 1.0) {
//      //continue;
//    }
     if (gl_LightSource[i].position.w <= 0.0) {
      // directional light
      const vec3 L = normalize(gl_LightSource[i].position.xyz);
      const vec3 R = normalize(-reflect(L, N));

      // diffuse
      Idiff = gl_FrontMaterial.diffuse.rgb * gl_LightSource[i].diffuse.rgb
              * max(dot(N, L), 0.0);

      // specular
      Ispec = gl_FrontMaterial.specular.rgb * gl_LightSource[i].specular.rgb
              * pow(max(dot(R, E), 0.0), safeShiny); //FIXME
    }
    else {
#endif // MULTITYPE

      const vec3 light = (gl_LightSource[i].position.xyz - V);
      const vec3 L = normalize(light);
      const vec3 R = normalize(-reflect(L,N));
      const float dist = length(light);
      const float atten = 1.0 / (gl_LightSource[i].constantAttenuation +
                                 (gl_LightSource[i].linearAttenuation * dist) +
                                 (gl_LightSource[i].quadraticAttenuation * dist * dist));
//      const float atten = 1.0 / (gl_LightSource[i].quadraticAttenuation * dist * dist);
                           
      // diffuse
      vec3 Tdiff = gl_FrontMaterial.diffuse.rgb * gl_LightSource[i].diffuse.rgb
                   * max(dot(N, L), 0.0);

      Idiff = Idiff + (atten * Tdiff);

      // specular
      vec3 Tspec = gl_FrontMaterial.specular.rgb * gl_LightSource[i].specular.rgb
                   * pow(max(dot(R, E), 0.0), safeShiny);
      Ispec = Ispec + (atten * Tspec);

#ifdef MULTITYPE
    }
#endif

#ifdef AMBIENT_ALPHA_TAG  
    }
#endif
  }

  gl_FragColor.rgb = (Itext0.rgb * Itext1.rgb * (gl_FrontLightModelProduct.sceneColor.rgb + Idiff))
                     + Ispec; // separate specular
  gl_FragColor.a = Itext0.a * gl_FrontMaterial.diffuse.a;
}


