
varying vec3 eye;
varying vec3 texspaceEye;

varying vec3 normal;
varying vec3 tangent;
varying vec3 binormal;

varying vec2 texCoord;

uniform float parallaxBias;
uniform float parallaxScale;

//const float parallaxScale = -0.03;

uniform sampler2D bzTexture0; // RGB + specular
uniform sampler2D bzTexture1; // normal + height


void main (void)
{

  const float shininess = gl_FrontMaterial.shininess;
  const vec3 matDiff = gl_FrontMaterial.diffuse.rgb;
  const vec3 matSpec = gl_FrontMaterial.specular.rgb;

  // parallax adjustment
  const float height = texture2D(bzTexture1, texCoord).a + parallaxBias;
  const vec3 tsEye = normalize(texspaceEye);
  texCoord  = texCoord - (tsEye.xy * (height * parallaxScale));

  // texture fetches
  vec4 diffuse = texture2D(bzTexture0, texCoord);
  vec3 tsNormal = texture2D(bzTexture1, texCoord).xyz * 2.0 - 1.0;
  
  const vec3 N = normalize((tsNormal.x * tangent) +
                           (tsNormal.y * binormal) +
                           (tsNormal.z * normal));
  const vec3 E = normalize(eye);
  

  // Tally the main light
  vec3 Idiff = vec3(0.0);
  vec3 Ispec = vec3(0.0);

#ifdef LAST_LIGHT
	// the main light
  const vec3 Lm = normalize(gl_LightSource[0].position.xyz);
  const vec3 Rm = -reflect(Lm, N);
  Idiff = gl_LightSource[0].diffuse.rgb * max(dot(N, Lm), 0.0);
  Ispec = gl_LightSource[0].specular.rgb * pow(max(dot(Rm, E), 0.0), shininess);

  // the point lights
  for (int i = 1; i <= LAST_LIGHT; i++) {
    if (gl_LightSource[i].ambient.a != 0.0) {
      const vec3 light = (gl_LightSource[i].position.xyz + eye);
      const float dist = length(light);
      const vec3 L = light / dist; // normalize
      const vec3 R = -reflect(L, N);

      const float atten =
				1.0 / (gl_LightSource[i].constantAttenuation +
               (gl_LightSource[i].linearAttenuation * dist) +
               (gl_LightSource[i].quadraticAttenuation * dist * dist));

      // diffuse
      const vec3 Tdiff = gl_LightSource[i].diffuse.rgb * max(dot(N, L), 0.0);
      Idiff = Idiff + (atten * Tdiff);

      // specular
      const vec3 Tspec = gl_LightSource[i].specular.rgb * pow(max(dot(R, E), 0.0), shininess);
      Ispec = Ispec + (atten * Tspec);
    }
	}
#endif // LAST_LIGHT
  
  const vec3 sceneColor = gl_FrontLightModelProduct.sceneColor.rgb;

  gl_FragColor.a   = gl_FrontMaterial.diffuse.a;
  gl_FragColor.rgb = (diffuse.a * Ispec * matSpec) +
                     (diffuse.rgb * (sceneColor + (Idiff * matDiff)));
}

